# frozen_string_literal: true

class Hermes::Api::Dmo::HousingRequestEntityFormsController < Hermes::Api::Dmo::ApplicationController
  set_instance_for :show_request

  class << self
    def mutate_params
      {
        consent_fields: {type: 'Array'},
        entity_form_level_id: {type: 'String'},
        level: {type: 'Integer'},
        delegation: {type: 'Boolean'},
        housing_request_id: {type: 'String'},
      }
    end 

    def acceptable_params
      {
        entity_id: {type: 'String'},
        applicant_id: {type: 'String'},
        delegation_signed: {type: 'Boolean'},
      }.merge(self.mutate_params).merge(super)
    end 

    def valid_operators
      %w[$in $nin $ne $gt $gte $lt $lte $eq $regex $all]
    end

    def regex_params
      {}
    end 
  end
  
  def index
    acceptable_params *self.class.acceptable_params.keys
    valid_operators *self.class.valid_operators
    super
  end

  def actives
    acceptable_params *self.class.acceptable_params.keys
    valid_operators *self.class.valid_operators

    paginate(
      index_aggregation.where(active: true),
      each_serializer: @serializer,
      **@options
    )
  end

  def show_request
    @instance = @model.find_by(id: params[:id])
    return render_json_error(:not_found, :instance_not_found) unless @instance

    entity = Hermes::Entity.readable_by(@user).find_by(id: @instance.entity_id)
    return render_json_error(:unauthorized, :not_reader) unless entity && current_user_roles.include?('entreprise')

    render json: @instance.consent_fields
  end

  def destroy
    return render_json_error(:not_found, :instance_not_found) unless @instance
    return render_json_error(:unauthorized, :not_owned) unless @instance.user_id == @user.id
    return render(json: @instance.errors, status: :unprocessable_entity) unless @instance.destroy

    render(body: nil, status: :no_content)
  end
end