# frozen_string_literal: true
require 'json'
require 'net/http'

# checker Api::Dmo::HermesController avant d'ajouter des methodes
class Hermes::Api::Dmo::HousingRequestsController < Hermes::Api::Dmo::ApplicationController
  hermes_tools 'main_user_id'
  hermes_errors 'housing_channel_error'

  class << self
    def mutate_params
      {
        actual_housing_details: {:type=>"Hash"},
        applicant_used_for_siret: {:type=>"String"},
        asked_mobility: {:type=>"Boolean"},
        attributed_housing_district_postal_code: {:type=>"String"},
        attributed_housing_district: {:type=>"String"},
        backoffice_status: {:type=>"String"},
        business_cotation: {:type=>"Float"},
        cdii: {:type=>"Boolean"},
        coapplicants: {:type=>"Array"},
        contract_type: {:type=>"String"},
        date_calculator_indicator: {:type=>"DateTime"},
        decider: {:type=>"String"},
        dependents_monthly_charges: {:type=>"Hash"},
        dependents_monthly_incomes: {:type=>"Hash"},
        dependents: {:type=>"Array"},
        disability_details: {:type=>"Array"},
        documents_required: {:type=>"Integer"},
        documents_uploaded: {:type=>"Integer"},
        ec_law_ressources: {:type=>"String"},
        electronical_renewal: {:type=>"Boolean"},
        employee_id: {:type=>"String"},
        employee_start_date: {:type=>"DateTime"},
        employee_unit: {:type=>"String"},
        entity_access_level: {:type=>"String"},
        entity_help_required: {:type=>"Boolean"},
        entity_refused: {:type=>"Boolean"},
        entity_validated: {:type=>"Boolean"},
        expulsion_judgement_date: {:type=>"Date"},
        forcing_duplicate: {:type=>"Boolean"},
        form_status: {:type=>"String"},
        has_multiple_attributions_under_conditions: {:type=>"Boolean"},
        housing_type_list: {:type=>"Array"},
        id_counter: {:type=>"String"},
        id_unique_associate: {:type=>"String"},
        id_unique: {:type=>"String"},
        internal_ref: {:type=>"String"},
        is_active: {:type=>"Boolean"},
        is_house_owner: {:type=>"Boolean"},
        is_long_delay_anormal: {:type=>"Boolean"},
        last_sne_renewal_date: {:type=>"Date"},
        lessor_name: {:type=>"String"},
        lessor_siren: {:type=>"String"},
        main_applicant_id: {:type=>"String"},
        mobility: {:type=>"Boolean"},
        motivations: {:type=>"Array"},
        nud: {:type=>"String"},
        other_reason: {:type=>"String"},
        owned_housing_details: {:type=>"Hash"},
        postal_details: {:type=>"Hash"},
        precisions: {:type=>"String"},
        priority_points_indexes: {:type=>"Array"},
        registration_number: {:type=>"String"},
        seeked_epci_first_quartils: {:type=>"Array"},
        seeked_housing_categories: {:type=>"Array"},
        seeked_localisation_list: {:type=>"Array"},
        seeked_max_rent: {:type=>"Integer"},
        seeked_quartil_belongings: {:type=>"Array"},
        seeked_types: {:type=>"Array"},
        seeking_elevator: {:type=>"Boolean"},
        seeking_first_floor: {:type=>"Boolean"},
        sne_created_at: {:type=>"DateTime"},
        sne_number: {:type=>"String"},
        social_housing_entry_date: {:type=>"DateTime"},
        social_housing_name: {:type=>"String"},
        tab_state: {:type=>"String"},
        test_created_at: {:type=>"DateTime"},
        unborn_children: {:type=>"Array"},
        valid_mobility: {:type=>"Boolean"},
        viewed_by_colaborator_offre_logement: {:type=>"Boolean"},
        widen_request: {:type=>"Boolean"},
        with_car_park: {:type=>"Boolean"},
        work_contract_type: {:type=>"Array"},
        workplace_insee_code: {:type=>"String"},
      }
    end 

    def acceptable_params
      {
        als_id: {:type=>"String"},
        applicant_used_for_siret_birth_date: {:type=>"Date"},
        applicant_used_for_siret_email: {:type=>"String"},
        applicant_used_for_siret_firstname: {:type=>"String"},
        applicant_used_for_siret_fullname: {:type=>"String"},
        applicant_used_for_siret_lastname: {:type=>"String"},
        applicants_attached_codes: {:type=>"Array"},
        category_type: {:type=>"Integer"},
        cerfa_version: {:type=>"String"},
        coapplicant_ids: {:type=>"Array"},
        consent_level: {:type=>"Integer"},
        cotations: {:type=>"Hash"},
        dalo: {:type=>"Boolean"},
        entity_id: {:type=>"String"},
        entity_status: {:type=>"String"},
        has_priority: {:type=>"Boolean"},
        housing_channel_keys: {:type=>"Array"},
        housing_offer_ids: {:type=>"Array"},
        is_striked_off: {:type=>"Boolean"},
        is_untensed_area: {:type=>"Boolean"},
        last_application_date: {:type=>"DateTime"},
        last_entity_code: {:type=>"String"},
        last_update_user_first_name: {:type=>"String"},
        last_update_user_id: {:type=>"String"},
        last_update_user_last_name: {:type=>"String"},
        min_surface: {:type=>"Integer"},
        monthly_revenues: {:type=>"Float"},
        nb_of_requests_made: {:type=>"Integer"},
        old_entity_status: {:type=>"String"},
        old_request_status: {:type=>"String"},
        profile_score: {:type=>"Integer"},
        reference_tax_revenues: {:type=>"Float"},
        request_status_updated_at: {:type=>"DateTime"},
        request_status: {:type=>"String"},
        requester_birth_date: {:type=>"Date"},
        requester_firstname: {:type=>"String"},
        requester_fullname: {:type=>"String"},
        requester_lastname: {:type=>"String"},
        seeked_departments: {:type=>"Array"},
        seeked_district_postal_codes: {:type=>"Array"},
        seeked_districts: {:type=>"Array"},
        single_parent: {:type=>"Boolean"},
        target_offer_reserving_business: {:type=>"String"},
        typology_range: {:type=>"Array"},
        unique_numbers: {:type=>"Array"},
      }.merge(self.mutate_params).merge(super)
    end 

    def valid_operators
      %w[$in $nin $ne $gt $gte $lt $lte $regex]
    end

    def regex_params
      {
        requester_firstname: {type: 'String'},
        requester_lastname: {type: 'String'},
        requester_fullname: {type: 'String'},
        applicant_used_for_siret_firstname: {type: 'String'},
        applicant_used_for_siret_lastname: {type: 'String'},
        applicant_used_for_siret_fullname: {type: 'String'},
      }
    end 
  end

  def include_relationships
    [:housing_applications]
  end

  def allow_true_user
    @allow_true_user = true
  end

  def regex_fullword_without_case_accent(param)
    if param && param[:$regex]
      param[:$regex] = /^#{::Regexp.escape(param[:$regex]).accent_to_regexp}/i
    end
  end

  def search
    index
  end

  def index
    acceptable_params *self.class.acceptable_params.keys
    regex_params *self.class.regex_params.keys
    valid_operators *self.class.valid_operators
    @query_precision ||= {}

    [
      :requester_firstname,
      :requester_lastname,
      :applicant_used_for_siret_firstname,
      :applicant_used_for_siret_lastname,
    ].each do |p|
      if params[p] && params[p][:$regex]
        params[p][:$regex] = /^#{::Regexp.escape(params[p][:$regex]).accent_to_regexp}/i
        @options[:hint] = "#{p}_1"
      end 
    end 

    if params[:main_employer_siret]
      applicants_ids = Hermes::Applicant.only(:id, :main_employer_siret).where(:main_employer_siret => {'$in' => params[:main_employer_siret]}).pluck(:id)
      @query_precision[:main_applicant_id] = {'$in': applicants_ids}
    end

    aggregation = index_aggregation

    if params[:requester_email]
      aggregation = aggregation.where(requester_email: /^#{Regexp.escape(params[:requester_email])}/i)
      @options[:hint]  = "requester_email_1"
    end
    
    paginate(
      aggregation,
      each_serializer: @serializer,
      **@options
    )
  end

  def entity_requests
    housing_request_id = params[:id]
    housing_request = Hermes::HousingRequest.find(housing_request_id)
    entity_requests_account = housing_request.entity_request_account
    render json: {status: "success", code: 200, message: {'entity_requests_account' => entity_requests_account}}
  end

  def entity_validation
    return render_json_error(:not_found, :instance_not_found) unless @instance
    return render_json_error(:unprocessable_entity, :not_entity_found) unless @instance.entity
    can_validate = is_super_admin || Hermes::UserRight.find_by(user: current_user, entity: @instance.entity, '$or': [ {right: 'user'}, { right: 'administrator'}])
    return render_json_error(:unauthorized, :not_authorized) unless can_validate
    return render_json_error(:unprocessable_entity, "attibutes_data_param_missing".to_sym) if !params.key?("data") || !params["data"].key?("attributes")

    %w[priority_points_indexes entity_validated entity_refused].each do |field|
      return render_json_error(:unprocessable_entity, "#{field}_param_missing".to_sym) unless params["data"]["attributes"].key?(field)
    end 

    unless @instance.update(
      priority_points_indexes: params["data"]["attributes"]["priority_points_indexes"]&.map{|a| a.to_i},
      entity_validated: params["data"]["attributes"]["entity_validated"],
      entity_refused: params["data"]["attributes"]["entity_refused"],
    )
      return respond_with_errors(@instance) 
    end

    render(
      jsonapi: @instance,
      serializer: @serializer,
      **@options
    )
  end 

  def spent_points
    housing_request_id = params[:id]
    housing_request = Hermes::HousingRequest.find(housing_request_id)
    entities_requests = housing_request.entity_request_account
    rented_requests = entities_requests.select {|request| request.request_rented == true}
    unless rented_requests.nil?
      spent_points = sum(rented_requests)
      render json: {status: "success", code: 200, message: {'spent_points' => spent_points}}
    end
  end
  
  def sum(requests)
    sum = 0
    requests.each do |request|
      sum += request["amount"].to_i
    end
    sum
  end

  def waiting_points
    housing_request_id = params[:id]
    housing_request = Hermes::HousingRequest.find(housing_request_id)
    entities_requests = housing_request.entity_request_account
    rented_requests = entities_requests.select {|request| request.request_rented == false}
    unless rented_requests.nil?
      waiting_points = sum(rented_requests)
      render json: {status: "success", code: 200, message: {'waiting_points' => waiting_points}}
    end
  end

  def is_a_collaborator_company
    current_user_roles.include?('collaborateur entreprise')
  end

  def is_a_collaborator_housing_offer
    current_user_roles.include?('collaborateur offre logement')
  end

  def entity_readable(housing_request)
    if Hermes::Entity.readable_by(@user).find_by(id: housing_request.entity.id)
      true
    else
      @error = " entity not readable by user"
      false
    end
  end

  def has_a_company_role
    if current_user_roles.include?('entreprise')
      true
    else
      @error = " user role doesn't include entreprise"
    end
  end

  def housing_request_match(housing_request)
    if housing_request.entity.id.to_s == params[:entity_id]
      true
    else
      @error = " housing request is not connected to this entity"
      false
    end
  end

  def has_an_entity(housing_request)
    if !housing_request.entity.nil?
      true
    else
      @error = "housing request has no entity"
      false
    end
  end

  def add_points_to_housing_request
    if !params[:points].nil? && !params[:entity_id].nil?
      if housing_request = Hermes::HousingRequest.find(params[:id])
        if has_an_entity(housing_request) && has_a_company_role && housing_request_match(housing_request) #&& entity_readable
          points = params[:points].to_i
          entity = Hermes::Entity.find_by(id: params[:entity_id])
          response_json = entity.create_entity_request_account(points, housing_request)
          render json: response_json
        else
          render json: {status: "unauthorized", code: 401, message: @error}
        end
      else
        render json: {status: "not found", code: 404, message: 'housing request not found'}
      end
    else
      render json: {status: "missing params", code: 401, message: 'some params are missing'}
    end
  end

  def entity_note
    set_instance
    if @instance
      if has_an_entity(@instance) && entity_readable(@instance) && current_user_roles.include?('cre')
        if @instance.entity_note.nil?
          render json: {status: "no content", code: 204, message: 'no entity note inside the housing request'}
        else
          render json: {status: "success", code: 200, message: @instance.entity_note}
        end
      else
        render json: {status: "unauthorized", code: 401, message: @error}
      end
    else
      render json: {status: "not found", code: 404, message: 'no housing request found'}
    end
  end

  def internal_note
    housing_request_id = params[:id]
    if !housing_request_id.nil?
      if is_a_collaborator_housing_offer
        housing_request = Hermes::HousingRequest.find(housing_request_id)
        if housing_request.nil?
          render json: {status: "not found", code: 404, message: 'no housing request found'}
        else
          if housing_request.internal_note.nil?
            render json: {status: "no content", code: 204, message: 'no internal note inside the housing request'}
          else
            render json: {status: "success", code: 200, message: housing_request.internal_note}
          end
        end
      else
        render json: {status: "unauthorized", code: 401, message: 'access denied user is no collaborator'}
      end
    else
      render json: {status: "missing param", code: 404, message: 'param is missing'}
    end
  end

  def update_entity_note
    set_instance
    new_entity_note = params[:entity_note]
    if !new_entity_note.nil?
      if @instance.nil?
        render json: {status: "not found", code: 404, message: 'no housing request found'}
      else
        if has_an_entity(@instance) && has_a_company_role
          sentence = bad_words_filter(new_entity_note)
          @instance.entity_note = sentence
          if @instance.save!
            render json: {status: "success", code: 200, message: {"message" => "successfullly modified", "value" => @instance.entity_note}}
          else
            render json: {status: "no content", code: 204, message: 'no internal note inside the housing request'}
          end
        end
      end
    else
      render json: {status: "missing params", code: 404, message: 'some params are missing'}
    end
  end

  def update_internal_note
    housing_request_id = params[:id]
    new_internal_note = params[:internal_note]
    if !housing_request_id.nil? && !new_internal_note.nil?
      if is_a_collaborator_housing_offer
        housing_request = Hermes::HousingRequest.find(housing_request_id)
        if housing_request.nil?
          render json: {status: "not found", code: 404, message: 'no housing request found'}
        else
          sentence = bad_words_filter(new_internal_note)
          housing_request.internal_note = sentence
          if housing_request.save!
            render json: {status: "success", code: 200, message: {"message" => "successfullly modified", "value" => housing_request.internal_note}}
          else
            render json: {status: "no content", code: 204, message: 'no internal note inside the housing request'}
          end
        end
      else
        render json: {status: "unauthorized", code: 401, message: 'access denied user is no collabroator'}
      end
    else
      render json: {status: "missing params", code: 404, message: 'some params are missing'}
    end
  end

  def bad_words_filter(sentence)
    filters = ["fuck", "fucking"] #to link with the model
    words = sentence.split(' ')
    words.size.times do |i|
      filters.each do |filter|
        clean_word = words[i].split(/\W+/).first
        if filter == clean_word
          words[i] = "***"
        end
      end
    end
    words.join(' ')
  end

  def has_application
    @instance.housing_offer_ids.kind_of?(Array) && @instance.housing_offer_ids.length > 0
  end

  def set_instance
    return unless params[:id]

    if params[:id] == 'active'
      if main_applicant_id = @user.main_applicant_id
        @instance = @model.readable_by(@user).find_by(main_applicant_id: main_applicant_id, is_active: true)
      end
    else 
      search_params = params[:id].length == 24 ? { id: params[:id]} : { als_id: params[:id]}
      instance = @model.find_by(search_params, **@query_precision)
      if instance && (instance.readable_by?(@user) || instance&.entity&.readable_by?(@user))
        @instance = instance
      else
        nil
      end
    end
  end

  def show
    return render_json_error(:not_found, :object_not_found) if !@instance
    eligible_count_option = params['options']&.include?('eligible_count')
    
    if !@instance.eligibility_query_base && eligible_count_option
      return render_json_error(:unprocessable_entity, :eligibility_query_base_missing) 
    end 
    
    @options = { eligible_count: eligible_count_option}

    super
  end

  def create
    @model.current_user_id = @user.id
    @additional_params = {main_user_id: main_user_id_param} if main_user_id_param
    # return render_error_not_mutable(not_mutable) unless not_mutable.empty?

    create_without_render
    update_last_updated_user_info

    return respond_with_errors(@instance) unless @instance.save

    render(jsonapi: @instance, serializer: @serializer, status: :created)
  end

  def update_last_updated_user_info
    @instance.last_update_user_id = @user.id.to_s
    @instance.last_update_user_last_name = @user.last_name
    @instance.last_update_user_first_name = @user.first_name
  end

  def update
    @model.current_user_id = @user.id
    @additional_params = {main_user_id: main_user_id_param} if main_user_id_param

    return render_json_error(:not_found, :object_not_found) if !@instance 
    # return render_error_not_mutable(not_mutable) unless not_mutable.empty?
    return render_json_error(:unprocessable_entity, :desactivation_avec_candidature) if mutate_valid_params[:is_active] === false && has_application
    return render_json_error(:unprocessable_entity, :cannot_validate) if params[:data][:attributes][:entity_validated].present? && !@instance.entity&.writable_by?(@user)
    
    @instance.with_lock(reload: false) do
      valid_params = mutate_valid_params
      
      param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
      valid_params[:user_id] = param_user if param_user && @user.has_role?('Administrator')
      return render_json_error(:parameters_not_present, :no_valid_parameters) unless valid_params.present?
      Rails.logger.debug "TEST valid_params are #{valid_params}"
      update_last_updated_user_info
      if @instance.update(valid_params) && @instance.save
        # Temporary_disabled_devops_7390
        # @instance.notify(key: NOTIFICATION_KEYS[:housing_request][:al_edit], sender: @user, channels: [])
      else
        return respond_with_errors(@instance)
      end

      render(jsonapi: @instance, serializer: @serializer)
    end
  end

  def update_next_renewal_date_set_by_sne
    return render_json_error(:unauthorized, :not_admin) unless current_user_roles.include?('Administrator') || is_super_admin
    return render_json_error(:not_found, :object_not_found) unless @instance 
    return render_json_error(:unprocessable_entity, :missing_params) unless params[:next_renewal_date_set_by_sne].present?

    @instance.update(next_renewal_date_set_by_sne: params[:next_renewal_date_set_by_sne] )

    render(jsonapi: @instance, serializer: @serializer)
  end 

  def update_from_sne
    hri = nil
    return render_json_error(:not_found, :object_not_found) if !@instance
    return render_json_error(:unprocessable_entity, :trying_to_update_an_inactive_request) unless @instance.is_active
    unless @instance.applicant_verified
      hri = Hermes::HousingRequestInitializer.where(done: false, unique_number: @instance.unique_numbers.last).order_by(created_at: :desc).first
      return render_json_error(:not_found, :housing_request_initilizer_not_found) unless hri
      return render_json_error(:unprocessable_entity, :pin_code_missing) if params["pin_code"].blank? 
      return render_json_error(:unprocessable_entity, :wrong_pin_code) unless hri.check_pin_code(params["pin_code"])
      @instance.set(applicant_verified: true, updated_at: DateTime.now)
    end 

    @model.current_user_id = @user.id
    return render_json_error(:not_found, :object_not_found) if !@instance

    result = @instance.update_request_from_sne(current_user) 

    if result.is_a? Hash
      render json: result, status: 422 # a voir comment utilsier render_json_error pour donner une liste d'erreur
    else 
      render(
        jsonapi: @instance.reload,
        serializer: @serializer,
        **@options
      )
    end
  end

  def force_update_renewal_values
    return render_json_error(:not_found, :object_not_found) unless @instance 

    result = @instance.update_renewal_values_from_sne(current_user) 

    if result.is_a? Hash
      render json: result, status: 422 
    else 
      render(
        jsonapi: @instance.reload,
        serializer: @serializer,
        **@options
      )
    end
  end

  def destroy
    return render_json_error(:not_found, :object_not_found) if !@instance
    return render_json_error(:unprocessable_entity, :suppression_avec_candidature) if has_application

    super
  end

  def admin_search
    return render json: {status: "unauthorized", code: 401, message: "user is no superadmin"} unless is_super_admin

    aggregation = Hermes::HousingRequest.all

    if !params['entity_params'].blank?
      entity_query = admin_search_param(
        params: params['entity_params'], 
        possible_params: [:id, :id_cristal, :siren, :siret],
        partial_regex_params: [:title]
      )
      entity_hint = entity_query["title"] ? 'title_1' : nil
      aggregation = aggregation.where(:entity_id.in => Hermes::Entity.where(entity_query).hint(entity_hint).pluck(:id)) 
    end 

    if !params['applicant_params'].blank?
      applicant_query = admin_search_param(
        params: params['applicant_params'], 
        possible_params: [:main_applicant_id, :requester_email],
        full_regex_params: [:requester_firstname, :requester_lastname]
      )
      @options[:hint] = 'requester_firstname_1' if applicant_query["requester_firstname"]
      @options[:hint] = 'requester_lastname_1' if applicant_query["requester_lastname"]
      aggregation = aggregation.where(applicant_query)
    end 

    if params[:unique_numbers].present?
      unique_numbers = params[:unique_numbers].is_a?(Array) ? params[:unique_numbers] : [params[:unique_numbers]]
      aggregation = aggregation.where( unique_numbers: { :$in => unique_numbers }) 
    end 

    @options[:no_attributes] = true

    paginate(
      aggregation,
      each_serializer: Hermes::AdminSearchRequestSerializer,
      **@options
    )
  end

  def clp_details
    return render_json_error(:missing_parameter, :als_ids_param_is_missing) unless params[:als_ids].present?

    aggregation = index_aggregation

    @options = { eligible_offers_count: true }
    
    if params[:reserving_siren].present?
      @options[:reserving_sirens] =  Hermes::ReservationHolder.find_by(siren: params[:reserving_siren])&.siren
    end 

    paginate(
      aggregation.where(:als_id.in => (params[:als_ids] || [])),
      each_serializer: @serializer,
      **@options
    )
  end

  def housing_request_folder
    render_json_error(:not_found, :object_not_found) unless @instance
    hrf = @instance.housing_request_folder
    #Rails.cache.fetch(hrf.cache_key, expires_in: 1.days) do
      render(
        jsonapi: hrf,
        serializer: Hermes::HousingRequestFolderSerializer,
        base_url: request.host,
        **@options
      )
    #end
  end


  def change_confirmed
    return render_json_error(:not_found, :object_not_found) unless @instance
    return render_json_error(:unprocessable_entity, :missing_params) if params[:required_documents].nil? || !params[:required_documents].is_a?(Hash)

    params[:required_documents].each do |key, value|
      if key == "all"
        @instance.housing_request_folder.required_documents.each do |rd|
          rd.update(declared: value)
        end 
        break
      end 
      Hermes::RequiredDocument.find(key)&.update(declared: value)
    end 

    render(
      jsonapi: @instance.housing_request_folder.reload,
      serializer: Hermes::HousingRequestFolderSerializer,
      **@options
    )
  end 

  def send_flux_thirty  
    array_of_ids = params[:ids].to_array
    array_of_als_ids = params[:als_ids].to_array
    return render_json_error(:unprocessable_entity, :wrong_params) if (array_of_ids.length + array_of_als_ids.length).zero?
    total = 0
    ids = array_of_ids
    cond = {}
    if ids.length > 0
      cond[:id.in] = ids
    else
      ids = array_of_als_ids
      cond[:als_id.in] = ids
    end
    ids.each do |id|
      if hr = Hermes::HousingRequest.find_by(cond)
        if hr.nuds && hr.nuds.length > 0
          hr.nuds.each do |nud|
            if nud.unique_number.length == 18
              total += 1
              so = Hermes::BoClient::AddRequestTask.new(housing_request_id: hr.id, nud_id: nud.id)
            end
          end
        elsif hr
          so = Hermes::BoClient::AddRequestTask.new(housing_request_id: hr.id)
        end
        if so
          total += 1
          ExecuteSopraTaskJob.perform_later(so.endpoint, so.to_xml, "post", "Hermes::HousingRequest", hr.id.to_s)
        end
      end
    end
    render body: {res: total.to_s + " housing_requests flux 30 sended to the workers"}.to_json, status: 200
  end

  def send_flux_thirty_sync
    if @instance
      if @instance.nuds && @instance.nuds.length > 0
        nud = @instance.nuds[0]
        if nud.unique_number.length == 18
          so = Hermes::BoClient::AddRequestTask.new(housing_request_id: @instance.id, nud_id: nud.id)
        end
      else
        so = Hermes::BoClient::AddRequestTask.new(housing_request_id: @instance.id)
      end
      if so
        res = ExecuteSopraTaskJob.perform_now(so.endpoint, so.to_xml, "post", "Hermes::HousingRequest", @instance.id.to_s)
      end
    end
    if res
      render body: {res: " housing_requests flux 30 sended with no errors"}.to_json, status: 200
    else
      if @instance.reload.xml_sopra_error_ids.last
      error = Hermes::XmlSopraError.find(@instance.reload.xml_sopra_error_ids.last)
      else
        error = "impossible de récuperer l'erreur"
        end
      render body: {error: error&.as_json}.to_json, status: 200
    end
  end

  def add_housing_channel
    keys = params[:keys] || [params[:key]]
    # If an array of keys is given, delete all existing keys to remplace by array
    if params[:keys].present?
      @instance.set(housing_channel_keys: [], updated_at: DateTime.current)
    end
    update_channel do
      keys.each do |key|
        if Hermes::HousingChannel.find_by(key: key)
          @instance.update_pending_channels!(key)
        else
          render_json_error(:not_found, :instance_not_found, {message: I18n.t('errors.housing_channel.not_found', key: key)}) and return
        end
      end
    end
  end

  def validate_housing_channel
    keys = params[:keys] || [params[:key]]
    update_channel do
      keys.each do |key|
        if Hermes::HousingChannel.find_by(key: key)
          @instance.update_validated_channels!(key)
        else
          render_json_error(:not_found, :instance_not_found, {message: I18n.t('errors.housing_channel.not_found', key: key)}) and return
        end
      end
    end
  end

  def remove_housing_channel
    keys = params[:keys] || [params[:key]]
    update_channel do
      keys.each do |key|
        if Hermes::HousingChannel.find_by(key: key)
          @instance.update_validated_channels!(key, false)
        else
          render_json_error(:not_found, :instance_not_found, {message: I18n.t('errors.housing_channel.not_found', key: key)}) and return
        end
      end
    end
  end

  def validate_code
    return render_json_error(:not_found, :instance_not_found) unless @instance
    return render_json_error(:not_acceptable, :missing_code) unless params.key?("code")

    if params[:code].nil?
      @instance.update(last_entity_code: nil)
      return render(jsonapi: @instance.reload, serializer: @serializer)
    end 

    entity_code = Hermes::EntityCode.find_by(code: params[:code], applicant_id: @instance.main_applicant_id)
    entity_code = Hermes::EntityCode.find_by(code: params[:code]) unless entity_code
    return render(json: {success: false, message: "Le code n'est pas valide."}, status: :not_found) unless entity_code

    if !entity_code.expired?
      applicant = Hermes::Applicant.find_by(id: @instance.main_applicant_id)
      return render_json_error(:not_found, :instance_not_found) unless applicant
      entity = Hermes::Entity.find(entity_code.entity_id)
      @instance.update(last_entity_code: entity_code.code)
      render(jsonapi: @instance.reload, serializer: @serializer)
    else
      render json: {
        success: false,
        message: 'Le code est expiré.'
      }, status: :unprocessable_entity
    end
  rescue ActionController::ParameterMissing => e
    render json: {success: false, message: e.message}, status: :not_acceptable
  end

  def notify
    return render_json_error(:not_found, :instance_not_found) unless @instance

    # mutate_valid_params include body and title (can includes channels array to)
    @notif = NotificationCenter::Notification.new(mutate_valid_params.merge({
                                                                              notifiable: @instance,
                                                                              targetable: @instance.main_applicant,
                                                                              sender: @user,
                                                                              key: NOTIFICATION_KEYS[:housing_request][:custom]
                                                                            }))

    if @notif.save
      render(
        jsonapi: @notif,
        serializer: ::Hermes::NotificationCenter::NotificationSerializer,
        status: :created
      )
    else
      render_json_error(:unprocessable_entity, :errors, {message: @notif.errors.full_messages})
    end
  end

  def send_message
    return render_json_error(:unauthorized, :not_als_collaborator) unless current_user_roles.include?('collaborateur offre logement')

    @doc = Doc.new
    @doc.assign_attributes(doc_params)
    @doc.type = 'email'
    @doc.responsible = @user.id
    @doc.bypass_refresh = true
    @doc.title = params[:name] if params[:name]
    @doc.title = params[:title] if params[:title]
    if @doc.save
      if to = params[:to] || (@doc.superfields && @doc.superfields['mail_to'])
        cc = params[:cc] || (@doc.superfields && @doc.superfields['mail_cc'])
        bcc = params[:bcc] || (@doc.superfields && @doc.superfields['mail_bcc'])
        from = 'no-reply@actionlogement.fr'
        if mail = Agent.regular_mail(@doc.id.to_s, to: to, title: @doc.title, from_no_reply: from, cc: cc, bcc: bcc, content: doc_params[:body])
          @doc.set(blob: mail.to_s, updated_at: DateTime.current)
          # @doc.get_email_infos

          mail.deliver_later
          render json: @doc, status: :created
        else
          render_json_error :unprocessable_entity, :sending_error
        end
      else
        render_json_error :unprocessable_entity, :missing_params_to
      end
    else
      respond_with_errors(@doc)
    end
  end

  def xml_sopra_errors
    return render_json_error(:not_found, :instance_not_found) unless @instance
    errors = Hermes::XmlSopraError.where(item_id: @instance.id, item_class: @instance.class).order_by(created_at: :desc)

    render(jsonapi: errors, each_serializer: Hermes::XmlSopraErrorSerializer)
  end 

  def config_change_minus_two
    render(json: { change_minus_two_date: HERMES_CONFIG['change_minus_two_date']&.to_s , change_minus_two_year: HERMES_CONFIG['change_minus_two_year']&.to_i }, status: 200)
  end 

  def eligible_offers
    return render_json_error(:not_found, :instance_not_found) unless @instance
    return render_json_error(:unprocessable_entity, :eligibility_query_base_missing) unless @instance.eligibility_query_base

    if params[:reserving_siren].present?
      rh = Hermes::ReservationHolder.find_by(siren: params[:reserving_siren])
      params[:reserving_siren] = { '$in' => rh.siren } if rh 
    end 

    acceptable_params *Hermes::Api::Dmo::HousingOffersController.acceptable_params.keys
    regex_params *Hermes::Api::Dmo::HousingOffersController.regex_params.keys
    valid_operators *Hermes::Api::Dmo::HousingOffersController.valid_operators
    options = {}

    eligibility_type = params[:eligibility_type] || 'department'
    other_query_params = { :$and => [ :$and => [build_query(Hermes::HousingOffer)]] }
    aggregation = @instance.eligibles_offers(eligibility_type).where(other_query_params).order_by(order_params)

    other_count = {}
    Hermes::HousingRequest.eligibility_types.each do |type|
      if params['options']&.include?("#{type}_count")
        eligibles_requests =  @instance.eligibles_offers(type).where(build_query(Hermes::HousingOffer))
        other_count["#{type}_count"] = @instance.eligibles_offers(type).where(build_query(Hermes::HousingOffer)).count(hint: @instance.eligibility_find_hint)
      end 
    end 
    options[:other_count] = other_count unless other_count.empty?
    options[:hint] =  @instance.eligibility_find_hint

    paginate(
      aggregation,
      each_serializer: Hermes::HousingOfferSerializer,
      **options
    )
  end

  def is_eligible_offer
    return render_json_error(:not_found, :instance_not_found) unless @instance
    return render_json_error(:not_acceptable, :missing_offer_id) if params[:offer_id].blank?
    return render_json_error(:not_found, :offer_not_found) unless offer = Hermes::HousingOffer.find(params[:offer_id])

    render json: { is_eligible: !!Hermes::HousingOffer.where(@instance.all_eligibility_query).find_by(id: offer.id.to_s) }
  end 

  private

  def update_channel
    return render_json_error(:not_found, :instance_not_found) unless @instance

    if (params[:keys].present? && params[:keys].is_a?(Array)) || params[:key].present?
      yield if block_given?

      render json: {
        pending_channels: @instance.housing_channel_pending_keys,
        validated_channels: @instance.housing_channel_keys
      }
    else
      raise ActionController::ParameterMissing.new('key or keys')
    end

  rescue ActionController::ParameterMissing => e
    render_json_error(:not_acceptable, :parameter_missing, {message: e.message})
  rescue Hermes::Errors::HousingChannelError => e
    code = e.message.include?("a déjà été ajouté")  ? :conflict : :unprocessable_entity
    render_json_error(code, :housing_channel_error, {message: e.message})
  end

  def entity_code_validation(entity_code, applicant_id)
    if !entity_code.expired?
      applicant = Hermes::Applicant.find_by(id: applicant_id)
      return render_json_error(:not_found, :instance_not_found) unless applicant

      applicant.update!(entity_id: entity_code.entity_id)
      @instance.update!(last_entity_code: entity_code.code)
      render(jsonapi: @instance, serializer: @serializer)
    else
      render_json_error(:unprocessable_entity, :expire_code, {message: 'Le code est expiré.'})
    end
  end

  def doc_params
    if params.key?(:data)
      ActiveModelSerializers::Deserialization.jsonapi_parse!(params, only: %i[
        title
        responsible
        body
        cycle_id
        family_id
        type
        percent
        bloc_name
        doc_ids
        template_id
        note
        writers
        readers
        consulted
        accounted
        followed_by
        type
        tagset_ids
        space_ids
        ask_of
        pin
        tags
        labels
        superfields
        event_slot_type
        event_location
        event_modality
        event_slots
      ], key_transform: :unaltered)
    end
  end
end
