# frozen_string_literal: true

Api::D2::UsersController.class_eval do

  RESTRICTED_FIELDS = [
    :id,
    :login,
    :current_auth_role,
    :created_at,
    :current_sign_in_at,
    :last_sign_in_at,
    :status_changed_at,
    :picture_id,
    :updated_at,
    :public_read,
    :writers,
    :readers,
    :inactive_at,
    :roles,
    :auth_roles,
    :first_name,
    :last_name,
    :email,
    :group_ids,
    :skill_ids,
    :status,
    :superfields,
    :tenants,
    :current_tenant,
    :type,
    :salutation,
    :phone_number,
    :cellular_telephone_number,
    :default_mail_account_id
  ]

  def index
    p = params.to_unsafe_h.except('controller', 'action', 'uuid', 'user', 'auth_token')
    if @user.trusted || @user.machine? || @user.has_role?("Administrator")
      per_page = 15
      page = 1

      if params[:page].is_a?(Hash)
        page = params[:page][:number].to_i if params[:page][:number]
        per_page = params[:page][:size].to_i if params[:page][:size]
      else
        per_page = params[:per_page].to_i if params[:per_page]
        page = params[:page].to_i if params[:page]
      end
      exact = (p.keys.size > 1) && p.has_key?(:exact)
      limit = params[:limit].blank? ? (exact ? 1 : 100) : params[:limit].to_i

      search_query = Search::UsersSearch.new(p, { user: @user, unscoped: true }).search
      user_query = {}

      if (online = params[:online]).present?
        if online.to_bool
          user_query[:channel_key.in] = $redis.hgetall('online_users').keys
        else
          user_query[:channel_key.nin] = $redis.hgetall('online_users').keys
        end
      end

      @users = search_query.where(user_query).limit(limit)
      @users = @users.where(id: @user.id) unless is_admin_or_super

      results = @users.only(RESTRICTED_FIELDS).sorter(params[:sort])
      count = @users.count

      paginate results, per_page: per_page, page: page
    else
      render json: {message: "you must call users api with manager account"}, status: :unauthorized

    end
  end

  def show
    return me if @user.id.to_s == params[:id]
    query = User.unscoped
    query = query.readable_by(@user) unless is_admin_or_super
    if @object = query.find_by(id: params[:id])
      render json: @object
    else
      render_json_error :not_found, :user_not_found
    end
  end

  def me
    if @user
      serializer = UserSerializer
      serializer = UserFullSerializer if params[:full] == 'true'
      render json: @user, serializer: serializer
    else
      render_json_error :not_found, :user_not_found
    end
  end

  private

  def is_admin_or_super 
    @user.has_role?("Administrator") || @user.auth_roles&.include?("superadministrateur")
  end
end
