require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::UntensedAreasController, :type => :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers
  include JwtAuthHelper

  check_mutate_params(
    model: Hermes::UntensedArea, 
    mutate_params: Hermes::Api::Dmo::UntensedAreasController.mutate_params, 
    factory: :untensed_area,
    user_roles: ['Administrator']
  )

  let!(:responsible) {create(:user, roles: ['Administrator'])}
  let!(:user) {create(:user)}

  #TESTS BASIQUES D'API

  describe 'POST /untensed_areas' do
    describe 'Create untensed_area with minimal valid datas' do
      before :each do
        authenticated_header(request, responsible)
        post 'create', {
            data: {
                attributes: {
                }
            }
        }
      end
      it 'returns status code 201' do
        expect(response).to have_http_status(201)
      end
    end
  end

  describe 'GET /untensed_areas' do
    describe 'GET /untensed_areas' do
      let!(:untensed_area1) {create(:untensed_area)}
      let!(:untensed_area3) {create(:untensed_area)}
      let!(:untensed_area2) {create(:untensed_area)}

      before :each do
        authenticated_header(request, responsible)
        get 'index'
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(3)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(3)
      end
    end

  end

  describe 'GET /untensed_areas/:id' do
    describe 'verif it work' do
      let!(:untensed_area1) {create(:untensed_area)}
      before :each do
        authenticated_header(request, responsible)
        get 'show', {id: untensed_area1.id}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
      end
    end
    describe 'verif it work not work with a non authorized user' do
      let!(:untensed_area1) {create(:untensed_area)}

      before :each do
        authenticated_header(request, user)
        get 'show', {id: untensed_area1.id}
      end
      it 'returns status code 401' do
        expect(response).to have_http_status(401)
      end
    end
  end

  describe 'PATCH /untensed_areas/:id' do

  end

  describe 'DELETE /untensed_areas/:id' do
    let!(:untensed_area) {create(:untensed_area, user_id: responsible.id.to_s)}
    before :each do
      authenticated_header(request, responsible)

      delete 'destroy', {
          id: untensed_area.id.to_s
      }
    end
    it 'returns status code 204' do
      expect(response).to have_http_status(204)
    end
  end

end

require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::UntensedAreasController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  tags = 'API_DMO_UNTENSED_AREAS'
  include JwtAuthHelper
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:admin) { create(:user, roles: ['Administrator']) }

  path '/api/dmo/untensed_areas' do
    get 'Retrieves all untensed_areas' do
      tags tags
      produces 'application/json'

      Hermes::Api::Dmo::UntensedAreasController.acceptable_params.sort.each do |params_name, options|
        parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
      end

      security [Bearer: {}]
      paginator

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }

        run_test!
      end 

      response '200', 'untensed_areas found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let!(:untensed_areas) { create_list(:untensed_area, 5) }

        include_context 'with integration test'
        include_context 'valid jsonapi collection' do
          let(:expected_collection) { untensed_areas }
        end
      end
    end
  end

  path '/api/dmo/untensed_areas' do
    post 'Create untensed_area' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::UntensedAreasController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )
      let(:params) do
        jsonapi_attributes(
          {  insee_code: 'test' }
        )
      end

      response '201', 'untensed_area created' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'

        after do |_example|
          json = JSON.parse(response.body)

          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'untensed_areas'
        end
      end
    end
  end

  path '/api/dmo/untensed_areas/{id}' do
    put 'Update untensed_area' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::UntensedAreasController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter name: :id, in: :path, type: :string
      let(:id) { create(:untensed_area, user: admin).id }
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )
      let(:params) do
        jsonapi_attributes(
          {  insee_code: 'test' }
        )
      end

      response '200', 'untensed_area updated' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'

        after do |_example|
          json = JSON.parse(response.body)
          
          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'untensed_areas'
        end
      end
    end
  end

  path '/api/dmo/untensed_areas/{id}' do
    delete 'Delete a single untensed_area by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      response '204', 'untensed_area deleted' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { create(:untensed_area).id }
        run_test!
      end
    end
  end

end