require 'rails_helper'

describe Api::D2::UsersController, type: :api do
  include Devise::Test::ControllerHelpers
  include JwtAuthHelper
  let!(:admin) { FactoryBot.create(:administrator) }
  let!(:superadministrateur) { FactoryBot.create(:user, auth_roles: ['superadministrateur']) }

  let!(:basic) { FactoryBot.create(:user) }

  let!(:user1) { FactoryBot.create(:user) }
  let!(:user2) { FactoryBot.create(:user) }
  let!(:user3) { FactoryBot.create(:user) }

  describe 'GET /users' do

    ["admin", "superadministrateur"].each do |user|
      it "works as #{user}" do
        authenticated_header(request, eval(user))
        get 'index'
        expect(response).to have_http_status(200)
        expect(json).not_to be_empty  
        expect(json['data'].length).to eq(6)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(6)
      end
    end

    it 'works as user1' do
      authenticated_header(request, user1)
      get 'index'
      expect(response).to have_http_status(200)
      expect(json).not_to be_empty  
      expect(json['data'].length).to eq(1)
      expect(json['meta']['pagination']).to be_present
      expect(json['meta']['pagination']['total_objects']).to eq(1)
    end
  end

  describe 'GET /users/:id' do
    ["admin", "superadministrateur"].each do |user|
      it "works as #{user}" do
        authenticated_header(request, eval(user))
        get 'show', id: user2.id
        expect(response).to have_http_status(200)
      end
    end

    it 'works as user1' do
      authenticated_header(request, user1)
      get 'show', id: user2.id
      expect(response).to have_http_status(404)
    end
  end 


  describe 'GET /users/me' do
    ["admin", "superadministrateur", "user1"].each do |user|
      it "works as #{user}" do
        authenticated_header(request, eval(user))
        get 'me'
        expect(response).to have_http_status(200)
      end
    end
  end 
end 
